class HealthBar {
    constructor(scene, name = "", width, height, x, y, valueMax, color, colorMin = COLOR_RED, middle = true) {
        this.bar = new Phaser.GameObjects.Graphics(scene);

        this.width = Math.floor(width);
        this.height = Math.floor(height);
        this.valueMax = valueMax;
        this.value = valueMax;
        this.color = color;
        this.colorMin = colorMin;

        this.draw();

        scene.add.existing(this.bar);

        height = height - 4;
        let style = {font: height.toString() + 'px stencil', fill: "#000000"};
        this.nameText = scene.add.text(x, y, name, style).setOrigin(0, 0);
        this.nameText.alpha = 0.5;

        this.setX(x, middle);
        this.setY(y);
    }

    setValue(value) {
        if (value < 0) {
            value = 0;
        }
        if (value > this.valueMax) {
            value = this.valueMax;
        }
        if (value !== this.value) {
            this.value = value;
            this.draw();
        }
    }

    setValueMax(valueMax = this.valueMax) {
        if (valueMax <= 0) {
            return;
        }
        let diff = valueMax - this.valueMax;
        this.valueMax = valueMax;
        this.setValue(this.value + diff);
    }

    increaseValueMax(amount = 0, color = this.color) {
        this.color = color;
        this.setValueMax(this.valueMax + amount);
    }

    decreaseValueMax(amount = 0, color = this.color) {
        this.color = color;
        this.setValueMax(this.valueMax - amount);
    }

    decrease(amount) {
        this.value -= amount;

        let diff = 0;
        if (this.value < 0) {
            diff = -this.value;
            this.value = 0;
        }

        this.draw();

        return diff;
    }

    getBar() {
        return this.bar;
    }

    increase(amount) {
        this.value += amount;
        if (this.value > this.valueMax) {
            this.value = this.valueMax;
        }
        this.draw();
    }

    setX(x, middle = true) {
        if (middle)
            x = x - this.width / 2;
        this.bar.setX(x);
        this.nameText.setX(x + 5);
    }

    setY(y) {
        this.bar.setY(y);
        this.nameText.setY(y);
    }

    draw() {
        this.bar.clear();

        //  BG
        this.bar.fillStyle(0x000000);
        this.bar.fillRect(0, 0, this.width, this.height);

        //  Health

        this.bar.fillStyle(0xffffff);
        this.bar.fillRect(2, 2, this.width - 4, this.height - 4);

        let percent = this.value / this.valueMax;

        if (percent < 1 / 3) {
            this.bar.fillStyle(this.colorMin);
        } else {
            this.bar.fillStyle(this.color);
        }

        let d = Math.floor(percent * (this.width - 4));

        this.bar.fillRect(2, 2, d, this.height - 4);
    }

    destroy() {
        this.bar.destroy();
        this.nameText.destroy();
    }

    getJson() {
        let json = '{';
        json += toJson("value", this.value) + toJson("valueMax", this.valueMax) + toJson("color", this.color) + toJson("colorMin", this.colorMin);
        json = endLineJson(json);
        json += '}';
        return json;
    }
}